@@ -30,6 +30,7 @@
 
 #include "gtkhtml-types.h"
 #include "gtkhtml-enums.h"
+#include <gtk/gtk.h>
 
 /* FIXME we should remove html dep */
 #include "htmltypes.h"
@@ -103,7 +104,7 @@ struct _GtkHTMLClass {
 	gboolean (* command)              (GtkHTML *html, GtkHTMLCommandType com_type);
 	void (* cursor_changed)       (GtkHTML *html);
 	void (* object_inserted)       (GtkHTML *html, int pos, int len);
-	void (* object_deleted)        (GtkHTML *html, int pos, int len);
+	void (* object_delete)        (GtkHTML *html, int pos, int len);
 
 	/* properties */
 	GtkHTMLClassProperties *properties;
@@ -187,6 +188,8 @@ gboolean                   gtk_html_export                        (GtkHTML
 								   const char                *type,
 								   GtkHTMLSaveReceiverFn      receiver,
 								   gpointer                   data);
+char *                     gtk_html_get_selection_html            (GtkHTML                   *html,
+								   int                       *len);
 
 /* Editable support.  */
 void                       gtk_html_set_editable                  (GtkHTML                   *html,
@@ -228,6 +231,19 @@ gint                       gtk_html_print_get_pages_num           (GtkHTML
 								   gdouble                    footer_height);
 void                       gtk_html_print_set_master              (GtkHTML                   *html,
 								   GnomePrintJob             *print_master);
+void 			   gtk_html_print_page_with_header_footer  (GtkHTML		     *html,
+								   GtkPrintContext 	     *print_context,
+			   					   gdouble		      header_height,
+								   gdouble		      footer_height,
+								   GtkHTMLPrintCallback       header_print,
+							           GtkHTMLPrintCallback       footer_print,
+								   gpointer 		      user_data);
+void			   gtk_html_print_page			   (GtkHTML		      *html,
+								    GtkPrintContext	      *print_context);
+gint 			   gtk_html_print_page_get_pages_num 	  (GtkHTML		      *html,
+								   GtkPrintContext            *print_context,
+					 			   gdouble		       header_height,
+								   gdouble                     footer_height);
 
 /* Title.  */
 const gchar               *gtk_html_get_title                     (GtkHTML                   *html);
@@ -312,6 +328,12 @@ void                       gtk_html_set_images_blocking           (GtkHTML
 								   gboolean                   block);
 gboolean                   gtk_html_has_undo                      (GtkHTML                   *html);
 void                       gtk_html_drop_undo                     (GtkHTML                   *html);
+char *                     gtk_html_get_url_at                    (GtkHTML                   *html,
+								   int                        x,
+								   int                        y);
+char *                     gtk_html_get_cursor_url                (GtkHTML                   *html);
+void                       gtk_html_set_tokenizer                 (GtkHTML                   *html,
+								   HTMLTokenizer             *tokenizer);
 
 /* DEPRECATED */
 #if 1
@@ -327,6 +349,7 @@ gboolean                   gtk_html_save                          (GtkHTML
 GtkHTMLStream             *gtk_html_begin_content                 (GtkHTML                   *html,
 								   gchar                     *content_type);
 void                       gtk_html_drag_dest_set                 (GtkHTML                   *html);
+
 #endif
 
 #endif /* _GTKHTML_H_ */
